import createPluggableMiddleware from '../lib/createPluggableMiddleware';

describe('createPluggableMiddleware', () => {
  it("uses argument as initial middleware", () => {
    let store = {
      dispatch(action) {}
    };
    spyOn(store, 'dispatch');
    let middleware = createPluggableMiddleware(store => next => action => store.dispatch({type: 'test'}));
    middleware(store)(() => {})({type: 'ignored'});
    expect(store.dispatch.calls.allArgs()).toEqual([[{type: 'test'}]]);
  });
  it("uses new middleware provided to replaceMiddleware", () => {
    let store = {
      dispatch(action) {}
    };
    spyOn(store, 'dispatch');
    let middleware = createPluggableMiddleware(store => next => action => store.dispatch({type: 'test'}));
    middleware.replaceMiddleware(store => next => action => store.dispatch({type: 'test2'}));
    middleware(store)(() => {})({type: 'ignored'});
    expect(store.dispatch.calls.allArgs()).toEqual([[{type: 'test2'}]]);
  });
  it("calls next(action) when middleware is null/undefined", () => {
    let middleware = createPluggableMiddleware();
    let store = {
      dispatch(action) {}
    };
    let next = jasmine.createSpy('next');
    middleware(store)(next)({type: 'action'});
    expect(next.calls.allArgs()).toEqual([[{type: 'action'}]]);
  });
});
